@import url("./bulma-0.7.5.css");

textarea {
  font-family: Consolas, Monaco, Lucida Console, Liberation Mono, DejaVu Sans Mono, Bitstream Vera Sans Mono, Courier New, monospace;
}

#output pre {
  padding: 0;
}